@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: nasl_plugins.c 25468 2016-06-08 19:50:28Z hdoreau $
+* $Id$
 * Description: Launches NASL plugins.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -55,35 +55,6 @@
 #include "processes.h"
 #include "log.h"
 
-/**
- * @brief Add a nvti's preferences to the global preferences.
- *
- * @param nvti  NVTI pointer.
- */
-static void
-prefs_add_nvti (const nvti_t *nvti)
-{
-  unsigned int i;
-
-  if (!nvti)
-    return;
-
-  for (i = 0; i < nvti_pref_len (nvti); i++)
-    {
-      char *pref, *cname;
-      const nvtpref_t *np = nvti_pref (nvti, i);
-
-      cname = g_strdup (nvtpref_name (np));
-      g_strchomp (cname);
-      pref = g_strdup_printf ("%s[%s]:%s", nvti_name (nvti), nvtpref_type (np),
-                              cname);
-      prefs_set (pref, nvtpref_default (np));
-
-      g_free (cname);
-      g_free (pref);
-    }
-}
-
 /**
  * @brief Add *one* .nasl plugin to the plugin list.
  *
@@ -94,33 +65,31 @@ prefs_add_nvti (const nvti_t *nvti)
  * arglist, it will be replaced.
  *
  * @param folder  Path to the plugin folder.
- * @param name    File-name of the plugin
+ * @param filename  File-name of the plugin
  *
  * @return 0 on success, -1 on error.
  */
 int
-nasl_plugin_add (char *folder, char *name)
+nasl_plugin_add (char *folder, char *filename)
 {
   char fullname[PATH_MAX + 1];
   int nasl_mode;
   nasl_mode = NASL_EXEC_DESCR;
-  nvti_t *nvti;
 
-  snprintf (fullname, sizeof (fullname), "%s/%s", folder, name);
+  snprintf (fullname, sizeof (fullname), "%s/%s", folder, filename);
 
   if (prefs_get_bool ("nasl_no_signature_check"))
     {
       nasl_mode |= NASL_ALWAYS_SIGNED;
     }
 
-  nvti = nvticache_get (name);
-  if (nvti == NULL)
+  if (!nvticache_check (filename))
     {
       nvti_t *new_nvti;
       struct arglist *plugin_args;
 
-      g_free (nvti);
       plugin_args = g_malloc0 (sizeof (struct arglist));
+      arg_add_value (plugin_args, "key", ARG_PTR, nvticache_get_kb ());
       new_nvti = nvti_new ();
       arg_add_value (plugin_args, "NVTI", ARG_PTR, new_nvti);
 
@@ -148,35 +117,14 @@ nasl_plugin_add (char *folder, char *name)
             log_write ("The timestamp for %s is from the future and could not be fixed.", fullname);
         }
 
-      if (nvti_oid (new_nvti) != NULL)
-        {
-          nvticache_add (new_nvti, name);
-          nvti = nvticache_get (name);
-        }
+      if (nvti_oid (new_nvti))
+        nvticache_add (new_nvti, filename);
       else
         // Most likely an exit was hit before the description could be parsed.
         log_write ("\r%s could not be added to the cache and is likely to stay"
-                   " invisible to the client.", name);
+                   " invisible to the client.", filename);
       nvti_free (new_nvti);
     }
-
-  if (nvti == NULL)
-    {
-      /* Discard invalid plugins */
-      log_write ("%s: Failed to load", name);
-      return -1;
-    }
-
-  if (nvti_oid (nvti) == NULL)
-    {
-      /* Discard invalid plugins */
-      log_write ("%s: Failed to load, no OID", name);
-      nvti_free (nvti);
-      return -1;
-    }
-  prefs_add_nvti (nvti);
-
-  nvti_free (nvti);
   return 0;
 }
 
