@@ -132,6 +132,7 @@ static openvassd_option openvassd_defaults[] = {
   {"report_host_details", "yes"},
   {"kb_location", KB_PATH_DEFAULT},
   {"timeout_retry", "3"},
+  {"time_between_request", "0"},
   {NULL, NULL}
 };
 
@@ -273,7 +274,6 @@ static pid_t
 loading_handler_start ()
 {
   pid_t child_pid, parent_pid;
-  int opts;
 
   init_loading_shm ();
   parent_pid = getpid ();
@@ -283,17 +283,6 @@ loading_handler_start ()
 
   proctitle_set ("openvassd (Loading Handler)");
   openvas_signal (SIGTERM, handle_loading_stop_signal);
-  if ((opts = fcntl (global_iana_socket, F_GETFL, 0)) < 0)
-    {
-      log_write ("fcntl: %s", strerror (errno));
-      exit (0);
-    }
-
-  if (fcntl (global_iana_socket, F_SETFL, opts | O_NONBLOCK) < 0)
-    {
-      log_write ("fcntl: %s", strerror (errno));
-      exit (0);
-    }
 
   /*
    * Forked process will handle client requests until parent dies or stops it
@@ -304,18 +293,44 @@ loading_handler_start ()
       unsigned int lg_address;
       struct sockaddr_un address;
       int soc;
+      fd_set set;
+      struct timeval timeout;
+      int rv, ret;
+      pid_t child_pid1;
 
       if (loading_stop_signal || kill (parent_pid, 0) < 0)
         break;
       lg_address = sizeof (struct sockaddr_un);
-      soc = accept (global_iana_socket, (struct sockaddr *) (&address),
-                    &lg_address);
-      loading_client_handle (soc);
-      sleep (1);
-    }
-  if (fcntl (global_iana_socket, F_SETFL, opts) < 0)
-    log_write ("fcntl: %s", strerror (errno));
 
+      if (listen (global_iana_socket, 5) < 0)
+        continue;
+
+      FD_ZERO(&set);
+      FD_SET(global_iana_socket, &set);
+
+      timeout.tv_sec = 0;
+      timeout.tv_usec = 500000;
+
+      rv = select(global_iana_socket + 1, &set, NULL, NULL, &timeout);
+      if(rv == -1) /* Select error. */
+        continue;
+      else if(rv == 0) /* Timeout. */
+        continue;
+      else
+        soc = accept (global_iana_socket, (struct sockaddr *) (&address),
+                    &lg_address);
+      if (soc == -1)
+        continue;
+
+      child_pid1 = fork ();
+      if (child_pid1 == 0)
+        {
+          loading_client_handle (soc);
+          close (soc);
+          exit (0);
+        }
+      waitpid (child_pid1, &ret, WNOHANG);
+    }
   exit (0);
 }
 
