@@ -28,6 +28,7 @@
 #include <stdlib.h> /* for atoi() */
 #include <string.h> /* for strcmp() */
 #include <stdio.h>  /* for snprintf() */
+#include <regex.h>  /* for regcomp() */
 
 #include <openvas/misc/prefs.h>          /* for prefs_get() */
 #include <openvas/base/nvticache.h>
@@ -49,25 +50,21 @@ extern int kb_get_port_state_proto (kb_t, int, char *);
  * @return Whether a port in a port list is closed or not.
  */
 static int
-get_closed_ports (kb_t kb, char *ports_list)
+get_closed_ports (kb_t kb, char *ports_list, char *proto)
 {
   int i;
   char **ports;
 
   if (!ports_list)
     return -1;
-
   ports = g_strsplit (ports_list, ", ", 0);
   for (i = 0; ports[i] != NULL; i ++)
     {
       int iport = atoi (ports[i]);
-      if (iport != 0)
+      if (iport > 0 && kb_get_port_state_proto (kb, iport, proto) != 0)
         {
-          if (kb_get_port_state_proto (kb, iport, "tcp") != 0)
-            {
-              g_strfreev (ports);
-              return iport;
-            }
+          g_strfreev (ports);
+          return iport;
         }
       else
         {
@@ -83,34 +80,6 @@ get_closed_ports (kb_t kb, char *ports_list)
 }
 
 
-/**
- * @brief Returns whether a port in a port list is closed or not.
- */
-static int
-get_closed_udp_ports (kb_t kb, char *ports_list)
-{
-  int i;
-  char **ports;
-
-  if (!ports_list)
-    return -1;
-  ports = g_strsplit (ports_list, ", ", 0);
-  if (!ports_list)
-    return -1;
-
-  for (i = 0; ports[i] != NULL; i ++)
-    {
-      int iport = atoi (ports[i]);
-      if (iport > 0 && kb_get_port_state_proto (kb, iport, "udp"))
-        {
-          g_strfreev (ports);
-          return iport;
-        }
-    }
-  g_strfreev (ports);
-  return 0;                     /* found nothing */
-}
-
 /**********************************************************
 
  		   Public Functions
@@ -245,6 +214,77 @@ kb_present_keyname_of_namelist (kb_t kb, char *keys, char **keyname)
   return 0;
 }
 
+/**
+ * @brief Checks mandatory keys presence and value in the KB.
+ * @param[in]   kb      KB handle where to search for the keys.
+ * @param[in]   keys    Comma separated list of mandatory keys.
+ *
+ * @return 1 if a key is missing or not matching its value, 0 otherwise.
+ */
+static int
+check_mandatory_keys (kb_t kb, char *keys)
+{
+  int i;
+  char **keynames;
+
+  if (!kb || !keys || !*keys)
+    return 0;
+  keynames = g_strsplit (keys, ", ", 0);
+  if (!keynames)
+    return 0;
+  for (i = 0; keynames[i] != NULL; i ++)
+    {
+      struct kb_item *kbi;
+      char *re_str = NULL, *pos;
+
+      /* Split, if key requires RE matching. */
+      if ((pos = strstr (keynames[i], "=")))
+        {
+          re_str = pos + 1;
+          *pos = '\0';
+        }
+
+      kbi = kb_item_get_single (kb, keynames[i], KB_TYPE_UNSPEC);
+      if (!kbi)
+        {
+          g_strfreev (keynames);
+          return 1;
+        }
+
+      if (re_str)
+        {
+          regex_t re;
+
+          /* Check if RE matches. */
+          if (kbi->type != KB_TYPE_STR || !kbi->v_str)
+            {
+              g_strfreev (keynames);
+              kb_item_free (kbi);
+              return 1;
+            }
+          if (regcomp (&re, re_str, REG_EXTENDED | REG_NOSUB | REG_ICASE))
+            {
+              g_warning ("Couldn't compile regex %s", re_str);
+              g_strfreev (keynames);
+              kb_item_free (kbi);
+              return 1;
+            }
+          if (regexec (&re, kbi->v_str, 0, NULL, 0) == REG_NOMATCH)
+            {
+              g_strfreev (keynames);
+              kb_item_free (kbi);
+              regfree (&re);
+              return 1;
+            }
+          regfree (&re);
+        }
+      kb_item_free (kbi);
+    }
+
+  g_strfreev (keynames);
+  return 0;
+}
+
 /**
  * @brief Check whether mandatory requirements for plugin are met.
  *
@@ -263,7 +303,7 @@ mandatory_requirements_met (kb_t kb,
   int ret;
 
   mandatory_keys = nvticache_get_mandatory_keys (plugin->oid);
-  ret = kb_missing_keyname_of_namelist (kb, mandatory_keys, NULL);
+  ret = check_mandatory_keys (kb, mandatory_keys);
 
   g_free (mandatory_keys);
   if (ret)
@@ -288,7 +328,7 @@ requirements_plugin (kb_t kb, struct scheduler_plugin *plugin)
    */
   error[sizeof (error) - 1] = '\0';
   tcp = nvticache_get_required_ports (plugin->oid);
-  if (tcp && *tcp && (get_closed_ports (kb, tcp)) == 0)
+  if (tcp && *tcp && (get_closed_ports (kb, tcp, "tcp")) == 0)
     {
       strncpy (error, "none of the required tcp ports are open",
                sizeof (error) - 1);
@@ -298,7 +338,7 @@ requirements_plugin (kb_t kb, struct scheduler_plugin *plugin)
   g_free (tcp);
 
   udp = nvticache_get_required_udp_ports (plugin->oid);
-  if (udp && *udp && (get_closed_udp_ports (kb, udp)) == 0)
+  if (udp && *udp && (get_closed_ports (kb, udp, "udp")) == 0)
     {
       strncpy (error, "none of the required udp ports are open",
                sizeof (error) - 1);
